/*
 * Rick van der Zwet <info@rickvanderzwet.nl>
 * License BSDLike http://rickvanderzwet.nl/LICENSE
 */

package nl.rickvanderzwet.farmville;

import java.awt.AWTException;
import java.awt.Color;
import java.awt.Point;
import java.awt.PointerInfo;
import java.awt.Robot;
import java.awt.event.InputEvent;
import java.util.Random;
import java.awt.MouseInfo;
import java.io.IOException;



public class Automator {
	
	public enum Direction {
	    UP, DOWN, LEFT, RIGHT
	}
	
	/* Constants */
	static Integer FARMSIZE_Y = 7;
	static Integer FARMSIZE_X = 22;
	static boolean mouseClickEnabled = false;

	
	private static Robot bot = null;
	private static Random randomGenerator = null;

	/* Parcel size in pixels */
	static Integer XMOVE = 22;	 
	static Integer YMOVE = 11; 
	
	private static void pressMouse() throws InterruptedException {
		if (mouseClickEnabled) {
			bot.mousePress(InputEvent.BUTTON1_MASK);
			// Constant values are tricky, computers can only do that
			Thread.sleep(100 + randomGenerator.nextInt(250));
			bot.mouseRelease(InputEvent.BUTTON1_MASK);
		}
		
	}
	
	private static void moveMouse(Direction direction) throws InterruptedException {
		moveMouse(direction, 100);
	}
	
	private static void moveMouse(Direction direction, Integer sleep_msec) throws InterruptedException {
		/* Relocate to next field */
		Point mouse = MouseInfo.getPointerInfo().getLocation();
		
		/* Grumble half pixels moves are not possible, this will hack around it */
		XMOVE = (XMOVE == 22) ? XMOVE-- : XMOVE++;
		YMOVE = (YMOVE == 11) ? YMOVE-- : YMOVE++;


		
		Point newLocation = mouse;
		switch (direction) {
			case UP:
				newLocation.x -= XMOVE;
				newLocation.y -= YMOVE;
    			break;
			case DOWN:
				newLocation.x += XMOVE;
				newLocation.y += YMOVE;
    			break;
			case LEFT:
				newLocation.x -= XMOVE;
				newLocation.y += YMOVE;
    			break;
			case RIGHT:
				newLocation.x += XMOVE;
				newLocation.y -= YMOVE;
    			break;
		}
		bot.mouseMove(newLocation.x, newLocation.y);
		Thread.sleep(sleep_msec);
		
		mouse = MouseInfo.getPointerInfo().getLocation();
		if (newLocation.x != mouse.x || newLocation.y != mouse.y) {
			System.out.println("Forced exit, mouse moved externally");
			System.exit(1);
		}

	}
	
	/**
	 * @param args
	 * @throws InterruptedException 
	 * @throws IOException 
	 */
	public static void main(String[] args) throws InterruptedException, IOException {
		try {
			//bot.mouseMove(100, 100);
			bot = new Robot();
			PointerInfo pointer = null;
			Point mouse = null;
		    randomGenerator = new Random();
			// Allow time to focus Firefox window
			Thread.sleep(3000);

			/*
			bot.mouseMove(500, 500);
			bot.mousePress(InputEvent.BUTTON1_DOWN_MASK);
			bot.mouseMove(700, 700);
			bot.mouseRelease(InputEvent.BUTTON1_MASK);
			*/
			
			/* 0 = up, 1 = down, 2 = left, 3 = right */
			
		    pressMouse();
		    
		    Direction direction = Direction.RIGHT;
		    
		    /* Move around naturally, from top left, to bottom right */
		    for (int i = 0; i < FARMSIZE_Y; i++) {
		    	for (int j = 0; j < FARMSIZE_X; j++) {
		    		pointer = MouseInfo.getPointerInfo();
					mouse = pointer.getLocation();
		    		System.out.println("x: " + mouse.x + ", y: " + mouse.y);


		    		/* Too difficult, try larger parts */
		    		Color p = bot.getPixelColor(mouse.x, mouse.y);	
		    		System.out.println("Pixel color R:" + p.getRed() + ",G:" + 
		    				p.getGreen() + ",B:" + p.getBlue());
		    		
		    		pressMouse();
		    		moveMouse(direction);
		    	}
		    
		    	moveMouse(Direction.DOWN);
		    	pressMouse();
		    	
		    	/* Flip direction around */
		    	direction = (direction == Direction.LEFT) ? Direction.RIGHT : Direction.LEFT;

		    }
		} catch (AWTException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		// TODO Auto-generated method stub

	}
}




